DROP FUNCTION USR_PROD_NAO_COMPRADO_PART
/

DROP TYPE USR_PROD_NAO_COMPRADO_PART_TB
/

DROP TYPE USR_PROD_NAO_CMP_PART_TYPE
/

CREATE TYPE USR_PROD_NAO_CMP_PART_TYPE AS OBJECT
  (CDFILIAL       VARCHAR2(4),
   NMFILIAL       VARCHAR2(30),
   DTINICIAL      DATE,
   DTFINAL        DATE,
   CDOPERADOR     VARCHAR2(12),
   NMOPERADOR     VARCHAR2(40),
   CDARVPROD      VARCHAR2(20),
   NMPRODUTO      VARCHAR2(50),
   CDREGIAO       VARCHAR2(3),
   NMREGIAO       VARCHAR2(30),
   IDSITUCOLETA   CHAR(1),
   REFORFIL       CHAR(1),
   IDAPRMARPRO    CHAR(1),
   CDQUALIF       CHAR(1),
   QFOPRODFORN    CHAR(1),
   PRCMPFILI      CHAR(1),
   PRNCMP         CHAR(1),
   REFOREG        CHAR(1),
   COTADO         CHAR(1),
   PARTFILI       CHAR(100),
   CDMARCA        CHAR(1000),
   CDFORNECED     VARCHAR2(1000),
   DTINICOT       DATE,
   DTFINCOT       DATE
  )
/

CREATE TYPE USR_PROD_NAO_COMPRADO_PART_TB AS TABLE OF USR_PROD_NAO_CMP_PART_TYPE
/

CREATE OR REPLACE FUNCTION USR_PROD_NAO_COMPRADO_PART
                          (P_CDFILIAL     IN VARCHAR2,
                           P_CDFILCENTCMP IN VARCHAR2,
                           P_CDREGIAO     IN VARCHAR2,
                           P_CDCOMPRADOR  IN VARCHAR2,
                           P_DTINICIAL    IN DATE,
                           P_DTFINAL      IN DATE,
                           P_DTINICOT     IN DATE,
                           P_DTFINCOT     IN DATE,
                           P_CDPRODINI    IN VARCHAR2,
                           P_CDPRODFIN    IN VARCHAR2)
                           RETURN USR_PROD_NAO_COMPRADO_PART_TB PIPELINED IS
   OUT_RECORD USR_PROD_NAO_CMP_PART_TYPE :=
              USR_PROD_NAO_CMP_PART_TYPE (NULL, NULL, NULL, NULL, NULL,
                                          NULL, NULL, NULL, NULL, NULL,
                                          NULL, NULL, NULL, NULL, NULL,
                                          NULL, NULL, NULL, NULL, NULL,
                                          NULL, NULL, NULL, NULL);
   V_NMFILIAL         FILIAL.NMFILIAL%TYPE;
   V_NMREGIAO         REGIAO.NMREGIAO%TYPE;
   V_NMOPERADOR       OPERADOR.NMOPERADOR%TYPE;
   V_IDSITUCOLETA     NUMBER;
   V_REFORFIL         NUMBER;
   V_REFOREG          NUMBER;
   V_IDAPRMARPRO      NUMBER;
   V_CDQUALIF         NUMBER;
   V_QFOPRODFORN      NUMBER;
   V_PRNCMP           NUMBER;
   V_V_PRCMPFILI      NUMBER;
   V_PRCMPFILI        NUMBER;
   V_AUX_MO           NUMBER;
   V_AUX_MP           NUMBER;
   V_AUX_RM           NUMBER;
   V_AUX_FO           NUMBER;
   V_AUX_FP           NUMBER;
   V_AUX_RF           NUMBER;
   V_AUX_MFO          NUMBER;
   V_AUX_MFP          NUMBER;
   V_AUX_RMF          NUMBER;

   V_IDSITUCADA       CHAR(1);
   V_E_IDSITUCOLETA   CHAR(1);
   V_E_REFORFIL       CHAR(1);
   V_E_IDAPRMARPRO    CHAR(1);
   V_E_CDQUALIF       CHAR(1);
   V_E_QFOPRODFORN    CHAR(1);
   V_E_REFOREG        CHAR(1);
   V_E_PRCMPFILI      CHAR(1);
   V_E_PRNCMP         CHAR(1);
   V_ERRO             CHAR(1);
   V_ERRO_PP          CHAR(1);
BEGIN
   /*INICIO DO C_PREVCONS QUE BUSCA DADOS DA FUNCAO USR_CONSUMO_PLANEJADO_PLA*/
   FOR C_PREVCONS IN (SELECT CDFILIAL, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO,
                             PRODUTO.CDPRODUTO, SUM(B.QTPREVCONS) QTPREVCONS,
                             MIN(DTPREVCONS) DTPREVCONS_MIN, MAX(DTPREVCONS) DTPREVCONS_MAX,
                             CDREGIAO, CDOPERADOR
                        FROM TABLE (USR_CONSUMO_PLANEJADO_PLA
                                                 (P_CDFILIAL,
                                                  P_CDFILCENTCMP,
                                                  P_CDREGIAO,
                                                  P_CDCOMPRADOR,
                                                  P_DTINICIAL,
                                                  P_DTFINAL,
                                                  0,
                                                  P_CDPRODINI,
                                                  P_CDPRODFIN)) B, PRODUTO, PRODCOMP, PRODUTO PRODUTOCMP
                       WHERE B.CDPRODUTO = PRODUTO.CDPRODUTO
                         AND PRODCOMP.CDPRODUTO = PRODUTOCMP.CDPRODUTO
                         AND ((INSTR(P_CDCOMPRADOR, PRODCOMP.CDOPERADOR) > 0) OR (P_CDCOMPRADOR = 'T'))
                         AND PRODUTO.CDARVPROD LIKE PRODUTOCMP.CDARVPROD||'%'
                       GROUP BY CDFILIAL, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO,
                                PRODUTO.CDPRODUTO, CDREGIAO, CDOPERADOR
                       ORDER BY CDFILIAL, CDOPERADOR, CDARVPROD, DTPREVCONS_MIN)
   LOOP
      SELECT MAX(NMFILIAL)
        INTO V_NMFILIAL
        FROM FILIAL
       WHERE CDFILIAL = C_PREVCONS.CDFILIAL;
      SELECT MAX(NMREGIAO)
        INTO V_NMREGIAO
        FROM REGIAO
       WHERE CDREGIAO = C_PREVCONS.CDREGIAO;
      SELECT MAX(NMOPERADOR)
        INTO V_NMOPERADOR
        FROM OPERADOR
       WHERE CDOPERADOR = C_PREVCONS.CDOPERADOR;

      V_IDSITUCOLETA   := 0;
      V_REFORFIL       := 0;
      V_REFOREG        := 0;
      V_PRCMPFILI      := 0;
      V_PRNCMP         := 0;
      V_E_IDSITUCOLETA := 'N';
      V_E_REFORFIL     := 'N';
      V_E_IDAPRMARPRO  := 'N';
      V_E_CDQUALIF     := 'N';
      V_E_QFOPRODFORN  := 'N';
      V_E_REFOREG      := 'N';
      V_E_PRCMPFILI    := 'N';
      V_E_PRNCMP       := 'N';
      V_ERRO           := 'S';
      V_ERRO_PP        := 'N';

      V_AUX_MO  := 0;
      V_AUX_MP  := 0;
      V_AUX_RM  := 0;
      V_AUX_FO  := 0;
      V_AUX_FP  := 0;
      V_AUX_RF  := 0;
      V_AUX_MFO := 0;
      V_AUX_MFP := 0;
      V_AUX_RMF := 0;

      OUT_RECORD.CDFILIAL     := C_PREVCONS.CDFILIAL;
      OUT_RECORD.NMFILIAL     := V_NMFILIAL;
      OUT_RECORD.DTINICIAL    := P_DTINICIAL;
      OUT_RECORD.DTFINAL      := P_DTFINAL;
      OUT_RECORD.CDOPERADOR   := C_PREVCONS.CDOPERADOR;
      OUT_RECORD.NMOPERADOR   := V_NMOPERADOR;
      OUT_RECORD.CDARVPROD    := C_PREVCONS.CDARVPROD;
      OUT_RECORD.NMPRODUTO    := C_PREVCONS.NMPRODUTO;
      OUT_RECORD.CDREGIAO     := C_PREVCONS.CDREGIAO;
      OUT_RECORD.NMREGIAO     := V_NMREGIAO;
      OUT_RECORD.IDSITUCOLETA := V_E_IDSITUCOLETA;
      OUT_RECORD.REFORFIL     := V_E_REFORFIL;
      OUT_RECORD.IDAPRMARPRO  := V_E_IDAPRMARPRO;
      OUT_RECORD.CDQUALIF     := V_E_CDQUALIF;
      OUT_RECORD.QFOPRODFORN  := V_E_QFOPRODFORN;
      OUT_RECORD.PRCMPFILI    := V_E_PRCMPFILI;
      OUT_RECORD.PRNCMP       := V_E_PRNCMP;
      OUT_RECORD.REFOREG      := V_E_REFOREG;
      OUT_RECORD.PARTFILI     := '';
      OUT_RECORD.CDMARCA      := '';
      OUT_RECORD.CDFORNECED   := '';
      OUT_RECORD.COTADO       := 'N';
      OUT_RECORD.DTINICOT     := P_DTINICOT;
      OUT_RECORD.DTFINCOT     := P_DTFINCOT;

      FOR C_PARTFILI IN (SELECT LISTAGG(CDTIPPART, ',') WITHIN GROUP (ORDER BY CDTIPPART) CDTIPPART,
                                LISTAGG(CDMARCA, ',') WITHIN GROUP (ORDER BY CDMARCA) CDMARCA,
                                LISTAGG(CDMARCA||CASE NVL(CDMARCA, 'X')
                                                   WHEN 'X'
                                                   THEN ''
                                                   ELSE CDTIPPART
                                                 END, ',') WITHIN GROUP (ORDER BY CDMARCA) LCDMARCA,
                                LISTAGG(CDFORNECED, ',') WITHIN GROUP (ORDER BY CDFORNECED)  CDFORNECED,
                                LISTAGG(CDFORNECED||CASE NVL(CDFORNECED, 'X')
                                                      WHEN 'X'
                                                      THEN ''
                                                      ELSE CDTIPPART
                                                    END, ',') WITHIN GROUP (ORDER BY CDFORNECED)  LCDFORNECED
                           FROM PARTFILI
                          WHERE NRCOLEPRECO IS NULL
                            AND CDFILIAL  = C_PREVCONS.CDFILIAL
                            AND CDPRODUTO = C_PREVCONS.CDPRODUTO) LOOP

         OUT_RECORD.PARTFILI   := C_PARTFILI.CDTIPPART;
         OUT_RECORD.CDMARCA    := C_PARTFILI.CDMARCA;
         OUT_RECORD.CDFORNECED := C_PARTFILI.CDFORNECED;
      END LOOP;

      /*INICIO DO C_ITEMCOLE QUE BUSCA DADOS DE ITENS DE COTAO*/
      FOR C_ITEMCOLE IN (SELECT DISTINCT ITEMCOLE.CDREGIAO, ITEMCOLE.CDPRODUTO, ITEMCOLE.CDMARCA,
                                ITEMCOLE.DTINIVALITM, ITEMCOLE.DTFINVALITM, PRODUTO.CDARVPROD,
                                ITEMCOLE.CDFORNECED, ITEMCOLE.NRCOLEPRECO
                           FROM COLETAPR, ITEMCOLE, PRODUTO
                          WHERE COLETAPR.CDFILIAL     = ITEMCOLE.CDFILIAL
                            AND COLETAPR.NRCOLEPRECO  = ITEMCOLE.NRCOLEPRECO
                            AND ITEMCOLE.CDFILIAL     = P_CDFILCENTCMP
                            AND ITEMCOLE.CDPRODUTO    = PRODUTO.CDPRODUTO
                            AND ITEMCOLE.CDREGIAO     = C_PREVCONS.CDREGIAO
                            AND COLETAPR.CDOPERADOR   = C_PREVCONS.CDOPERADOR
                            AND PRODUTO.CDPRODESTO    = C_PREVCONS.CDPRODUTO
                            AND ITEMCOLE.VRPRECUNIITM <> 0
                            AND ITEMCOLE.DTINIVALITM BETWEEN CASE
                                                                WHEN ITEMCOLE.DTINIVALITM < P_DTINICOT
                                                                THEN ITEMCOLE.DTINIVALITM
                                                                ELSE P_DTINICOT
                                                             END AND P_DTFINCOT
                            AND ITEMCOLE.DTFINVALITM BETWEEN P_DTINICOT AND CASE
                                                                               WHEN ITEMCOLE.DTFINVALITM > P_DTFINCOT
                                                                               THEN ITEMCOLE.DTFINVALITM
                                                                               ELSE P_DTFINCOT
                                                                            END) LOOP
         OUT_RECORD.PARTFILI   := '';
         OUT_RECORD.CDMARCA    := '';
         OUT_RECORD.CDFORNECED := '';
         V_ERRO := 'N'; /*TESTES PARA VERIFICAR EXISTENCIA DE ALGUM ERRO*/

         /*CONSULTA QUALIFICAO DE MARCA*/
         SELECT COUNT(CDFILIAL)
           INTO V_IDAPRMARPRO
           FROM (SELECT PARASUPRI.CDFILIAL
                   FROM COLETAPR, ITEMCOLE, QFOMARPROD, PARASUPRI
                  WHERE ITEMCOLE.CDFILIAL        = P_CDFILCENTCMP
                    AND ITEMCOLE.CDFORNECED      = C_ITEMCOLE.CDFORNECED
                    AND ITEMCOLE.CDREGIAO        = C_ITEMCOLE.CDREGIAO
                    AND ITEMCOLE.CDPRODUTO       = C_ITEMCOLE.CDPRODUTO
                    AND ITEMCOLE.CDMARCA         = C_ITEMCOLE.CDMARCA
                    AND C_ITEMCOLE.DTINIVALITM  <= ITEMCOLE.DTINIVALITM
                    AND C_ITEMCOLE.DTFINVALITM  >= ITEMCOLE.DTFINVALITM
                    AND COLETAPR.CDOPERADOR      = C_PREVCONS.CDOPERADOR
                    AND ITEMCOLE.VRPRECUNIITM   <> 0
                    AND COLETAPR.CDFILIAL        = ITEMCOLE.CDFILIAL
                    AND COLETAPR.NRCOLEPRECO     = ITEMCOLE.NRCOLEPRECO
                    AND (PARASUPRI.CDFILIAL      = ITEMCOLE.CDFILIAL)
                    AND (QFOMARPROD.CDPRODUTO    = ITEMCOLE.CDPRODUTO)
                    AND (QFOMARPROD.CDMARCA      = ITEMCOLE.CDMARCA)
                    AND ((QFOMARPROD.IDAPRMARPRO = 'N'  AND QFOMARPROD.IDAUTMARPRO = 'S')
                    OR  ((QFOMARPROD.IDAPRMARPRO = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M'
                    AND COLETAPR.DTCOLEPREC     <= LAST_DAY(QFOMARPROD.DTVALAPROVA))
                    OR  (PARASUPRI.IDULTIDTVALQ  = 'D'
                    AND COLETAPR.DTCOLEPREC     <= QFOMARPROD.DTVALAPROVA)) ) )
                  UNION ALL
                 SELECT PARASUPRI.CDFILIAL
                   FROM COLETAPR, ITEMCOLE, PARASUPRI
                  WHERE ITEMCOLE.CDFILIAL       = P_CDFILCENTCMP
                    AND ITEMCOLE.CDFORNECED     = C_ITEMCOLE.CDFORNECED
                    AND ITEMCOLE.CDREGIAO       = C_ITEMCOLE.CDREGIAO
                    AND ITEMCOLE.CDPRODUTO      = C_ITEMCOLE.CDPRODUTO
                    AND ITEMCOLE.CDMARCA        = C_ITEMCOLE.CDMARCA
                    AND C_ITEMCOLE.DTINIVALITM <= ITEMCOLE.DTINIVALITM
                    AND C_ITEMCOLE.DTFINVALITM >= ITEMCOLE.DTFINVALITM
                    AND COLETAPR.CDOPERADOR     = C_PREVCONS.CDOPERADOR
                    AND ITEMCOLE.VRPRECUNIITM  <> 0
                    AND COLETAPR.CDFILIAL       = ITEMCOLE.CDFILIAL
                    AND COLETAPR.NRCOLEPRECO    = ITEMCOLE.NRCOLEPRECO
                    AND (PARASUPRI.CDFILIAL     = P_CDFILCENTCMP
                    AND PARASUPRI.IDMARSEMQUA  <> 'S'
                    AND NOT EXISTS (SELECT QFOMARPROD.CDMARCA
                                      FROM QFOMARPROD
                                     WHERE (QFOMARPROD.CDPRODUTO = ITEMCOLE.CDPRODUTO)
                                       AND (QFOMARPROD.CDMARCA   = ITEMCOLE.CDMARCA))));
         /*GRAVA IDENTIFICADOR DE QUALIFICAO DE MARCA - QM*/
         IF V_IDAPRMARPRO = 0 THEN
            V_E_IDAPRMARPRO := 'S';
            V_ERRO          := 'S';
         END IF;
         /*CONSULTA QUALIFICAO DE FORNECEDOR*/
         SELECT COUNT(CDFILIAL)
           INTO V_CDQUALIF
           FROM (SELECT ITEMCOLE.CDFILIAL
                   FROM QUALIFICACAO, PARASUPRI, FORNECEDOR, COLETAPR, ITEMCOLE
                  WHERE ITEMCOLE.CDFILIAL       = P_CDFILCENTCMP
                    AND ITEMCOLE.CDFORNECED     = C_ITEMCOLE.CDFORNECED
                    AND ITEMCOLE.CDREGIAO       = C_ITEMCOLE.CDREGIAO
                    AND ITEMCOLE.CDPRODUTO      = C_ITEMCOLE.CDPRODUTO
                    AND ITEMCOLE.CDMARCA        = C_ITEMCOLE.CDMARCA
                    AND C_ITEMCOLE.DTINIVALITM <= ITEMCOLE.DTINIVALITM
                    AND C_ITEMCOLE.DTFINVALITM >= ITEMCOLE.DTFINVALITM
                    AND COLETAPR.CDOPERADOR     = C_PREVCONS.CDOPERADOR
                    AND ITEMCOLE.VRPRECUNIITM  <> 0
                    AND COLETAPR.CDFILIAL       = ITEMCOLE.CDFILIAL
                    AND COLETAPR.NRCOLEPRECO    = ITEMCOLE.NRCOLEPRECO
                    AND FORNECEDOR.CDFORNECED   = ITEMCOLE.CDFORNECED
                    AND PARASUPRI.CDFILIAL      = P_CDFILCENTCMP
                    AND FORNECEDOR.CDQUALIF     = QUALIFICACAO.CDQUALIF
                    AND (QUALIFICACAO.IDPERCOTCMP <> '2' AND (FORNECEDOR.IDFORNAUTO = 'S'
                    OR  ((PARASUPRI.IDULTIDTVALQ = 'D'   AND COLETAPR.DTCOLEPREC < FORNECEDOR.DTVALQUALIF)
                    OR  (PARASUPRI.IDULTIDTVALQ  = 'M'    AND COLETAPR.DTCOLEPREC < LAST_DAY(FORNECEDOR.DTVALQUALIF)))))
                  UNION ALL
                 SELECT ITEMCOLE.CDFILIAL
                   FROM PARASUPRI, FORNECEDOR, COLETAPR, ITEMCOLE
                  WHERE ITEMCOLE.CDFILIAL       = P_CDFILCENTCMP
                    AND ITEMCOLE.CDFORNECED     = C_ITEMCOLE.CDFORNECED
                    AND ITEMCOLE.CDREGIAO       = C_ITEMCOLE.CDREGIAO
                    AND ITEMCOLE.CDPRODUTO      = C_ITEMCOLE.CDPRODUTO
                    AND ITEMCOLE.CDMARCA        = C_ITEMCOLE.CDMARCA
                    AND C_ITEMCOLE.DTINIVALITM <= ITEMCOLE.DTINIVALITM
                    AND C_ITEMCOLE.DTFINVALITM >= ITEMCOLE.DTFINVALITM
                    AND COLETAPR.CDOPERADOR     = C_PREVCONS.CDOPERADOR
                    AND ITEMCOLE.VRPRECUNIITM  <> 0
                    AND COLETAPR.CDFILIAL       = ITEMCOLE.CDFILIAL
                    AND COLETAPR.NRCOLEPRECO    = ITEMCOLE.NRCOLEPRECO
                    AND FORNECEDOR.CDFORNECED   = ITEMCOLE.CDFORNECED
                    AND PARASUPRI.CDFILIAL      = ITEMCOLE.CDFILIAL
                    AND FORNECEDOR.CDQUALIF     IS NULL);
         /*GRAVA IDENTIFICADOR DE QUALIFICAO DE FORNECEDOR - QF*/
         IF V_CDQUALIF = 0 THEN
            V_E_CDQUALIF := 'S';
            V_ERRO       := 'S';
         END IF;

         /*CONSULTA QUALIFICAO DE PRDOUTO POR FORNECEDOR*/
         SELECT COUNT(ITEMCOLE.CDFILIAL)
           INTO V_QFOPRODFORN
           FROM COLETAPR, ITEMCOLE
          WHERE ITEMCOLE.CDFILIAL       = P_CDFILCENTCMP
            AND ITEMCOLE.CDFORNECED     = C_ITEMCOLE.CDFORNECED
            AND ITEMCOLE.CDREGIAO       = C_ITEMCOLE.CDREGIAO
            AND ITEMCOLE.CDPRODUTO      = C_ITEMCOLE.CDPRODUTO
            AND ITEMCOLE.CDMARCA        = C_ITEMCOLE.CDMARCA
            AND C_ITEMCOLE.DTINIVALITM <= ITEMCOLE.DTINIVALITM
            AND C_ITEMCOLE.DTFINVALITM >= ITEMCOLE.DTFINVALITM
            AND COLETAPR.CDOPERADOR     = C_PREVCONS.CDOPERADOR
            AND ITEMCOLE.VRPRECUNIITM  <> 0
            AND COLETAPR.CDFILIAL       = ITEMCOLE.CDFILIAL
            AND COLETAPR.NRCOLEPRECO    = ITEMCOLE.NRCOLEPRECO
            AND NOT EXISTS (SELECT QFOPRODFORN.CDPRODUTO
                              FROM QFOPRODFORN, PARASUPRI
                             WHERE PARASUPRI.CDFILIAL = ITEMCOLE.CDFILIAL
                               AND (QFOPRODFORN.CDPRODUTO  = ITEMCOLE.CDPRODUTO)
                               AND (QFOPRODFORN.CDFORNECED = ITEMCOLE.CDFORNECED)
                               AND ((QFOPRODFORN.IDFORNAPROV = 'N'  AND QFOPRODFORN.IDFORNAUTOR = 'N')
                               OR  ((QFOPRODFORN.IDFORNAPROV = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M'
                               AND COLETAPR.DTCOLEPREC > LAST_DAY(QFOPRODFORN.DTVALPRFORN)) OR (PARASUPRI.IDULTIDTVALQ = 'D'
                               AND COLETAPR.DTCOLEPREC > QFOPRODFORN.DTVALPRFORN)) ) ));
         /*GRAVA IDENTIFICADOR DE QUALIFICAO DE PRODUTO PO FORNECEDOR - QPF*/
         IF V_QFOPRODFORN = 0 THEN
            V_E_QFOPRODFORN := 'S';
            V_ERRO          := 'S';
         END IF;

         /*CONSULTA FORNECEDOR NO ATENDE FILIAL*/
         SELECT COUNT(1)
           INTO V_REFORFIL
           FROM (SELECT COLETAPR.IDSITUCOLETA
                   FROM COLETAPR, ITEMCOLE, REFORFIL
                  WHERE ITEMCOLE.CDFILIAL       = P_CDFILCENTCMP
                    AND ITEMCOLE.CDFORNECED     = C_ITEMCOLE.CDFORNECED
                    AND ITEMCOLE.CDREGIAO       = C_ITEMCOLE.CDREGIAO
                    AND ITEMCOLE.CDPRODUTO      = C_ITEMCOLE.CDPRODUTO
                    AND ITEMCOLE.CDMARCA        = C_ITEMCOLE.CDMARCA
                    AND ITEMCOLE.VRPRECUNIITM  <> 0
                    AND C_ITEMCOLE.DTINIVALITM <= ITEMCOLE.DTINIVALITM
                    AND C_ITEMCOLE.DTFINVALITM >= ITEMCOLE.DTFINVALITM
                    AND COLETAPR.CDFILIAL       = ITEMCOLE.CDFILIAL
                    AND COLETAPR.NRCOLEPRECO    = ITEMCOLE.NRCOLEPRECO
                    AND COLETAPR.CDOPERADOR     = C_PREVCONS.CDOPERADOR
                    AND REFORFIL.CDFILIAL       = C_PREVCONS.CDFILIAL
                    AND REFORFIL.CDFORNECED     = ITEMCOLE.CDFORNECED
                    AND REFORFIL.CDPERIENTR     IS NOT NULL
                 UNION ALL
                 SELECT COLETAPR.IDSITUCOLETA
                   FROM COLETAPR, ITEMCOLE
                  WHERE ITEMCOLE.CDFILIAL       = P_CDFILCENTCMP
                    AND ITEMCOLE.CDFORNECED     = C_ITEMCOLE.CDFORNECED
                    AND ITEMCOLE.CDREGIAO       = C_ITEMCOLE.CDREGIAO
                    AND ITEMCOLE.CDPRODUTO      = C_ITEMCOLE.CDPRODUTO
                    AND ITEMCOLE.CDMARCA        = C_ITEMCOLE.CDMARCA
                    AND C_ITEMCOLE.DTINIVALITM <= ITEMCOLE.DTINIVALITM
                    AND C_ITEMCOLE.DTFINVALITM >= ITEMCOLE.DTFINVALITM
                    AND COLETAPR.CDOPERADOR     = C_PREVCONS.CDOPERADOR
                    AND ITEMCOLE.VRPRECUNIITM  <> 0
                    AND COLETAPR.CDFILIAL       = ITEMCOLE.CDFILIAL
                    AND COLETAPR.NRCOLEPRECO    = ITEMCOLE.NRCOLEPRECO
                    AND NOT EXISTS (SELECT REFORFIL.CDFILIAL
                                      FROM REFORFIL
                                     WHERE REFORFIL.CDFILIAL   = C_PREVCONS.CDFILIAL
                                       AND REFORFIL.CDFORNECED = ITEMCOLE.CDFORNECED));
         /*GRAVA IDENTIFICADOR DE FORNECEDOR NO ATENDE A FILIAL - FNF*/
         IF V_REFORFIL = 0 THEN
            V_E_REFORFIL := 'S';
            V_ERRO       := 'S';
         END IF;

         /*CONSULTA FORNECEDOR NO ATENDE A REGIAO*/
         SELECT COUNT(1)
           INTO V_REFOREG
           FROM (SELECT COLETAPR.IDSITUCOLETA
                   FROM COLETAPR, ITEMCOLE, REFOREG
                  WHERE ITEMCOLE.CDFILIAL       = P_CDFILCENTCMP
                    AND ITEMCOLE.CDFORNECED     = C_ITEMCOLE.CDFORNECED
                    AND ITEMCOLE.CDREGIAO       = C_ITEMCOLE.CDREGIAO
                    AND ITEMCOLE.CDPRODUTO      = C_ITEMCOLE.CDPRODUTO
                    AND ITEMCOLE.CDMARCA        = C_ITEMCOLE.CDMARCA
                    AND ITEMCOLE.VRPRECUNIITM  <> 0
                    AND C_ITEMCOLE.DTINIVALITM <= ITEMCOLE.DTINIVALITM
                    AND C_ITEMCOLE.DTFINVALITM >= ITEMCOLE.DTFINVALITM
                    AND COLETAPR.CDFILIAL       = ITEMCOLE.CDFILIAL
                    AND COLETAPR.NRCOLEPRECO    = ITEMCOLE.NRCOLEPRECO
                    AND COLETAPR.CDOPERADOR     = C_PREVCONS.CDOPERADOR
                    AND REFOREG.CDREGIAO        = C_ITEMCOLE.CDREGIAO
                    AND REFOREG.CDFORNECED      = ITEMCOLE.CDFORNECED
                    AND REFOREG.CDPERIENTR      IS NOT NULL
                  UNION ALL
                 SELECT COLETAPR.IDSITUCOLETA
                   FROM COLETAPR, ITEMCOLE
                  WHERE ITEMCOLE.CDFILIAL       = P_CDFILCENTCMP
                    AND ITEMCOLE.CDFORNECED     = C_ITEMCOLE.CDFORNECED
                    AND ITEMCOLE.CDREGIAO       = C_ITEMCOLE.CDREGIAO
                    AND ITEMCOLE.CDPRODUTO      = C_ITEMCOLE.CDPRODUTO
                    AND ITEMCOLE.CDMARCA        = C_ITEMCOLE.CDMARCA
                    AND C_ITEMCOLE.DTINIVALITM <= ITEMCOLE.DTINIVALITM
                    AND C_ITEMCOLE.DTFINVALITM >= ITEMCOLE.DTFINVALITM
                    AND COLETAPR.CDOPERADOR     = C_PREVCONS.CDOPERADOR
                    AND ITEMCOLE.VRPRECUNIITM  <> 0
                    AND COLETAPR.CDFILIAL       = ITEMCOLE.CDFILIAL
                    AND COLETAPR.NRCOLEPRECO    = ITEMCOLE.NRCOLEPRECO
                    AND NOT EXISTS (SELECT REFOREG.CDREGIAO
                                      FROM REFOREG
                                     WHERE REFOREG.CDREGIAO   = C_PREVCONS.CDREGIAO
                                       AND REFOREG.CDFORNECED = ITEMCOLE.CDFORNECED));
         /*GRAVA IDENTIFICADOR DE FORNECEDOR NO ATENDE A FILIAL - FNR*/
         IF V_REFOREG = 0 THEN
            V_E_REFOREG := 'S';
            V_ERRO      := 'S';
         END IF;

         /*CONSULTA PRODUTO COMPRADO NA FILIAL*/
         SELECT COUNT(1)
           INTO V_PRCMPFILI
           FROM PARAMGERAL
          WHERE NOT EXISTS (SELECT 'S' AS PRCMPFILI
                              FROM PRODCMPFILI, PRODUTO P
                             WHERE PRODCMPFILI.CDPRODUTO = P.CDPRODUTO
                               AND PRODCMPFILI.CDFILIAL  = C_PREVCONS.CDFILIAL
                               AND C_ITEMCOLE.CDARVPROD LIKE P.CDARVPROD||'%'
                               AND P.CDARVPROD > (SELECT NVL(MAX(PR.CDARVPROD), 0)
                                                    FROM PRODNCMP, PRODUTO PR
                                                   WHERE PRODNCMP.CDPRODUTO = PR.CDPRODUTO
                                                     AND C_ITEMCOLE.CDARVPROD LIKE PR.CDARVPROD||'%'
                                                     AND PRODNCMP.CDFILIAL = C_PREVCONS.CDFILIAL));
         /*GRAVA IDENTIFICADOR DE PRODUTO COMPRADO NA FILIAL - PCF*/
         IF V_PRCMPFILI = 0 THEN
            V_E_PRCMPFILI := 'S';
            V_ERRO        := 'S';
         END IF;

         /*CONSULTA PRODUTO NO COMPRADO NA FILIAL*/
         SELECT COUNT(1)
           INTO V_PRNCMP
           FROM PRODNCMP, PRODUTO P
          WHERE
              PRODNCMP.CDPRODUTO = P.CDPRODUTO AND
              PRODNCMP.CDFILIAL = C_PREVCONS.CDFILIAL AND
              C_ITEMCOLE.CDARVPROD LIKE P.CDARVPROD||'%' AND
              P.CDARVPROD > (SELECT NVL(MAX(PR.CDARVPROD), '0')
                               FROM PRODCMPFILI, PRODUTO PR
                              WHERE PRODCMPFILI.CDPRODUTO = PR.CDPRODUTO AND
                                    C_ITEMCOLE.CDARVPROD LIKE PR.CDARVPROD||'%' AND
                                    PRODCMPFILI.CDFILIAL = C_PREVCONS.CDFILIAL);

         /*GRAVA IDENTIFICADOR DE PRODUTO NO COMPRADO NA FILIAL - PNF*/
         IF V_PRNCMP > 0 THEN
            V_E_PRNCMP := 'S';
            V_ERRO     := 'S';
         END IF;

         /*CONSULTA SITUACAO DA COTACAO*/
         SELECT COUNT(1)
           INTO V_IDSITUCOLETA
           FROM COLETAPR, ITEMCOLE
          WHERE ITEMCOLE.CDFILIAL       = P_CDFILCENTCMP
            AND ITEMCOLE.CDFORNECED     = C_ITEMCOLE.CDFORNECED
            AND ITEMCOLE.CDREGIAO       = C_ITEMCOLE.CDREGIAO
            AND ITEMCOLE.CDPRODUTO      = C_ITEMCOLE.CDPRODUTO
            AND ITEMCOLE.CDMARCA        = C_ITEMCOLE.CDMARCA
            AND ITEMCOLE.VRPRECUNIITM  <> 0
            AND C_ITEMCOLE.DTINIVALITM <= ITEMCOLE.DTINIVALITM
            AND C_ITEMCOLE.DTFINVALITM >= ITEMCOLE.DTFINVALITM
            AND COLETAPR.CDFILIAL       = ITEMCOLE.CDFILIAL
            AND COLETAPR.NRCOLEPRECO    = ITEMCOLE.NRCOLEPRECO
            AND COLETAPR.CDOPERADOR     = C_PREVCONS.CDOPERADOR
            AND NVL(COLETAPR.IDSITUCOLETA, 'C') = 'C';
         /*GRAVA IDENTIFICADOR DE SITUACAO DA COTACAO - AND*/
         IF V_IDSITUCOLETA = 0 THEN
            V_E_IDSITUCOLETA := 'S';
            V_ERRO           := 'S';
         END IF;

         /*INICIO DE CONSULTA DE PARTICULARIDADE*/
         V_ERRO_PP           := 'N';
         OUT_RECORD.PARTFILI := '';
         FOR C_PARTFILI IN (SELECT LISTAGG(CDTIPPART, ',') WITHIN GROUP (ORDER BY CDTIPPART) CDTIPPART,
                                   LISTAGG(CDMARCA, ',') WITHIN GROUP (ORDER BY CDMARCA) CDMARCA,
                                   LISTAGG(CDMARCA||CASE NVL(CDMARCA, 'X')
                                                      WHEN 'X'
                                                      THEN ''
                                                      ELSE CDTIPPART
                                                    END, ',') WITHIN GROUP (ORDER BY CDMARCA) LCDMARCA,
                                   LISTAGG(CDFORNECED, ',') WITHIN GROUP (ORDER BY CDFORNECED)  CDFORNECED,
                                   LISTAGG(CDFORNECED||CASE NVL(CDFORNECED, 'X')
                                                          WHEN 'X'
                                                          THEN ''
                                                          ELSE CDTIPPART
                                                       END, ',') WITHIN GROUP (ORDER BY CDFORNECED)  LCDFORNECED
                              FROM PARTFILI
                             WHERE NRCOLEPRECO IS NULL
                               AND CDFILIAL  = C_PREVCONS.CDFILIAL
                               AND CDPRODUTO = C_ITEMCOLE.CDPRODUTO) LOOP

            OUT_RECORD.PARTFILI := C_PARTFILI.CDTIPPART;
            IF INSTR(C_PARTFILI.CDTIPPART, '1') > 0  THEN
               /*GRAVA IDENTIFICADOR DE MARCA OBRIGATORIA - MO*/
               IF INSTR(C_PARTFILI.LCDMARCA, C_ITEMCOLE.CDMARCA||'1') = 0 THEN
                  OUT_RECORD.CDMARCA := C_PARTFILI.CDMARCA;
                  V_ERRO_PP := 'S';
               END IF;
            END IF;

            IF INSTR(C_PARTFILI.CDTIPPART, '2') > 0  THEN
               /*GRAVA IDENTIFICADOR DE MARCA PREFERENCIAL - MP*/
               IF INSTR(C_PARTFILI.LCDMARCA, C_ITEMCOLE.CDMARCA||'2') = 0 THEN
                  OUT_RECORD.CDMARCA := C_PARTFILI.CDMARCA;
                  V_ERRO_PP := 'S';
               END IF;
            END IF;

            IF INSTR(C_PARTFILI.CDTIPPART, '3') > 0  THEN
               /*GRAVA IDENTIFICADOR DE RESTIO A MARCA - RM*/
               IF INSTR(C_PARTFILI.LCDMARCA, C_ITEMCOLE.CDMARCA||'3') > 0 THEN
                  OUT_RECORD.CDMARCA := C_PARTFILI.CDMARCA;
                  V_ERRO_PP := 'S';
               END IF;
            END IF;

            IF INSTR(C_PARTFILI.CDTIPPART, '4') > 0  THEN
               /*GRAVA IDENTIFICADOR DE FORNECEDOR OBRIGATORIO - FO*/
               IF INSTR(C_PARTFILI.LCDFORNECED, C_ITEMCOLE.CDFORNECED||'4') = 0 THEN
                  OUT_RECORD.CDFORNECED := C_PARTFILI.CDFORNECED;
                  V_ERRO_PP := 'S';
               END IF;
            END IF;

            IF INSTR(C_PARTFILI.CDTIPPART, '5') > 0  THEN
               /*GRAVA IDENTIFICADOR DE FORNECEDOR PREFERENCIAO - FP*/
               IF INSTR(C_PARTFILI.LCDFORNECED, C_ITEMCOLE.CDFORNECED||'5') = 0 THEN
                  OUT_RECORD.CDFORNECED := C_PARTFILI.CDFORNECED;
                  V_ERRO_PP := 'S';
               END IF;
            END IF;
            IF INSTR(C_PARTFILI.CDTIPPART, '6') > 0 THEN
               /*GRAVA IDENTIFICADOR DE RESTIO AO FORNECEDOR - RF*/
               IF INSTR(C_PARTFILI.LCDFORNECED, C_ITEMCOLE.CDFORNECED||'6') > 0 THEN
                  OUT_RECORD.CDFORNECED := C_PARTFILI.CDFORNECED;
                  V_ERRO_PP := 'S';
               END IF;
            END IF;

            IF INSTR(C_PARTFILI.CDTIPPART, '7') > 0  THEN
               /*GRAVA IDENTIFICADOR DE MARCA/FORNECEDOR OBRIGATORIO - MFO*/
               IF ((INSTR(C_PARTFILI.LCDFORNECED, C_ITEMCOLE.CDFORNECED||'7') = 0) OR
                   (INSTR(C_PARTFILI.LCDMARCA, C_ITEMCOLE.CDMARCA||'7') = 0)) THEN
                  OUT_RECORD.CDMARCA    := C_PARTFILI.CDMARCA;
                  OUT_RECORD.CDFORNECED := C_PARTFILI.CDFORNECED;
                  V_ERRO_PP := 'S';
               END IF;
            END IF;

            IF INSTR(C_PARTFILI.CDTIPPART, '8') > 0  THEN
               /*GRAVA IDENTIFICADOR DE MARCA/FORNECEDOR PREFERENCIAL - MFP*/
               IF ((INSTR(C_PARTFILI.LCDFORNECED, C_ITEMCOLE.CDFORNECED||'8') = 0) OR
                   (INSTR(C_PARTFILI.LCDMARCA, C_ITEMCOLE.CDMARCA||'8') = 0)) THEN
                  OUT_RECORD.CDMARCA    := C_PARTFILI.CDMARCA;
                  OUT_RECORD.CDFORNECED := C_PARTFILI.CDFORNECED;
                  V_ERRO_PP := 'S';
               END IF;
            END IF;

            IF INSTR(C_PARTFILI.CDTIPPART, '9') > 0  THEN
               /*GRAVA IDENTIFICADOR DE RETIO A MARCA/FORNECEDOR RMF*/
               IF ((INSTR(C_PARTFILI.LCDFORNECED, C_ITEMCOLE.CDFORNECED||'9') > 0) OR
                   (INSTR(C_PARTFILI.LCDMARCA, C_ITEMCOLE.CDMARCA||'9') > 0)) THEN
                  OUT_RECORD.CDMARCA    := C_PARTFILI.CDMARCA;
                  OUT_RECORD.CDFORNECED := C_PARTFILI.CDFORNECED;
                  V_ERRO_PP := 'S';
               END IF;
            END IF;
            IF V_ERRO_PP = 'N' THEN
               OUT_RECORD.PARTFILI   := '';
               OUT_RECORD.CDMARCA    := '';
               OUT_RECORD.CDFORNECED := '';
            END IF;
         END LOOP;/*FIM DE CONSULTA DE PARTICULARIDADE*/

         IF ((V_ERRO = 'S') OR (V_ERRO_PP = 'S')) THEN
            /*GRAVA OS ERRROS ENCONTRADOS COMO NO COTADO*/
            OUT_RECORD.IDSITUCOLETA := V_E_IDSITUCOLETA;
            OUT_RECORD.REFORFIL     := V_E_REFORFIL;
            OUT_RECORD.IDAPRMARPRO  := V_E_IDAPRMARPRO;
            OUT_RECORD.CDQUALIF     := V_E_CDQUALIF;
            OUT_RECORD.QFOPRODFORN  := V_E_QFOPRODFORN;
            OUT_RECORD.PRCMPFILI    := V_E_PRCMPFILI;
            OUT_RECORD.PRNCMP       := V_E_PRNCMP;
            OUT_RECORD.REFOREG      := V_E_REFOREG;
            OUT_RECORD.COTADO       := 'N';
         ELSE
            /*GRAVA ITEM COTADO*/
            OUT_RECORD.PARTFILI     := '';
            OUT_RECORD.IDSITUCOLETA := 'N';
            OUT_RECORD.REFORFIL     := 'N';
            OUT_RECORD.IDAPRMARPRO  := 'N';
            OUT_RECORD.CDQUALIF     := 'N';
            OUT_RECORD.QFOPRODFORN  := 'N';
            OUT_RECORD.PRCMPFILI    := 'N';
            OUT_RECORD.PRNCMP       := 'N';
            OUT_RECORD.REFOREG      := 'N';
            OUT_RECORD.COTADO       := 'S';
            EXIT;
         END IF;
      END LOOP; /*FIM DO C_ITEMCOLE QUE BUSCA DADOS DE ITENS DE COTAO*/
      IF OUT_RECORD.COTADO = 'N' THEN
         PIPE ROW (OUT_RECORD);
      END IF;
   END LOOP; /*FIM DO C_PREVCONS QUE BUSCA DADOS DA FUNCAO USR_CONSUMO_PLANEJADO_PLA*/
   RETURN;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('USR_PROD_NAO_COMPRADO_PART','6.02.XXX'); END;
/
